<?php

namespace Drupal\better_unpublished_terms;

use Drupal\Core\Access\AccessResult;
use Drupal\Core\Entity\EntityPublishedInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Entity\EntityInterface;
use Drupal\taxonomy\TermAccessControlHandler;
use http\Exception\InvalidArgumentException;

/**
 * Provides an access control handler for taxonomy terms.
 *
 * This class extends the default TermAccessControlHandler to allow
 * unpublished terms to be viewed if the user has the appropriate permissions.
 */
class BetterTermAccessControlHandler extends TermAccessControlHandler {

  /**
   * {@inheritdoc}
   */
  protected function checkAccess(EntityInterface $entity, $operation, AccountInterface $account) {
    if ($account->hasPermission('administer taxonomy')) {
      return AccessResult::allowed()->cachePerPermissions();
    }

    /** @var \Drupal\Core\Entity\EntityPublishedInterface $entity */
    switch ($operation) {
      case 'view':
        if (!$entity instanceof EntityPublishedInterface) {
          throw new InvalidArgumentException("Entity does not implement EntityPublishedInterface interface.");
        }

        if ($entity->isPublished()) {
          $access_result = AccessResult::allowedIf($account->hasPermission('access content') && $entity->isPublished())
            ->cachePerPermissions()
            ->addCacheableDependency($entity);
          if (!$access_result->isAllowed()) {
            $access_result->setReason("The 'access content' permission is required and the taxonomy term must be published.");
          }
        }
        else {
          // If the term is unpublished check to see if the user has the permission to edit/create/delete terms in this vocabulary.
          // If they do, allow access to the unpublished term.
          $access_result = AccessResult::allowedIf($account->hasPermission("edit terms in {$entity->bundle()}") || $account->hasPermission("delete terms in {$entity->bundle()}") || $account->hasPermission("create terms in {$entity->bundle()}"))
            ->cachePerPermissions()
            ->addCacheableDependency($entity);

          if (!$access_result->isAllowed()) {
            $access_result->setReason("The 'edit terms in {$entity->bundle()}' or 'delete terms in {$entity->bundle()}' permission is required to view unpublished terms.");
          }
        }

        return $access_result;

      default:
        // For all other operations, we delegate to the parent class.
        return parent::checkAccess($entity, $operation, $account);
    }
  }

}
